#include <qdp-config.h>

int lattice_size[NDIM] = { 8, 8, 8, 16 };

int
main(int argc, char *argv[])
{
  int volume;
  int status = 1;
  int mu;
  QDP_ColorMatrix *U[NDIM];
  QLA_Real plaq;

  /* start QDP */
  QDP_initialize(&argc, &argv);

  /* set lattice size and create layout */
  QDP_set_latsize(NDIM, lattice_size);
  QDP_create_layout();

  /* compute lattice volume */
  for (volume = 1, mu = 0; mu < NDIM; mu++)
    volume *= QDP_coord_size(mu);

  /* allocate the gauge field and set it to unit */
  for (mu = 0; mu < NDIM; mu++) {
    QLA_Complex one;
    QLA_c_eq_r(one, 1.0);

    U[mu] = QDP_create_M();
    QDP_M_eq_c(U[mu], &one, QDP_all);
  }

  /* Compute plaquette */
  plaq = plaquette(U);

  /* delete the gauge field */
  for (mu = 0; mu < NDIM; mu++) QDP_destroy_M(U[mu]);  

  /* Display the value */
  printf0("plaquette{U=1} = %g\n",
           plaq / (volume * QDP_Nc * NDIM * (NDIM - 1) / 2 ));

  status = 0;
end:
  /* shutdown QDP */
  QDP_finalize();

  return status;
}
