#include <qdp-config.h>

int lattice_size[NDIM] = { 8, 8, 8, 16 };

int
main(int argc, char *argv[])
{
  int status = 1;
  int mu;
  QDP_ColorMatrix *U[NDIM];
  QDP_RandomState *random_state;
  QLA_Real plaq;

  /* start QDP */
  QDP_initialize(&argc, &argv);

  /* set lattice size and create layout */
  QDP_set_latsize(NDIM, lattice_size);
  QDP_create_layout();

  /* create random generator */
  random_state = QDP_create_S();
  /* seed it */
  init_random(random_state, 299792458);

  /* allocate the gauge field and make it random */
  for (mu = 0; mu < NDIM; mu++) {
    U[mu] = QDP_create_M();
    random_link(U[mu], random_state, QDP_all);
  }

  /* Compute plaquette */
  plaq = plaquette(U);

  /* delete random generator */
  QDP_destroy_S(random_state);

  /* delete the gauge field */
  for (mu = 0; mu < NDIM; mu++) QDP_destroy_M(U[mu]);  

  /* Display the value */
  printf0("plaquette{random link} = %g\n",
           plaq / (QDP_volume() * QDP_Nc * NDIM * (NDIM - 1) / 2 ));

  status = 0;
end:
  /* shutdown QDP */
  QDP_finalize();

  return status;
}
