#include <qdp-config.h>
#include <math.h>

#define SCALE 0.7

static QLA_Complex
det(QLA_ColorMatrix *m)
{
  QLA_ColorMatrix tm;
  QLA_Complex z1;
  int i, j, c;

  for(j = 0; j < QDP_Nc; j++)
    for(i = 0; i < QDP_Nc; i++)
      QLA_elem_M(tm,j,i) = QLA_elem_M(*m,j,i);

  for(j = 0; j < QDP_Nc; j++) {
    for(i = 0; i <= j; i++) {
      QLA_Complex t2;
      t2 = QLA_elem_M(tm,j,i);
      for(c = 0; c < i; c++)
        QLA_c_meq_c_times_c(t2, QLA_elem_M(tm,c,i), QLA_elem_M(tm,j,c));

      QLA_elem_M(tm,j,i) = t2;
    }

    for(i = (j+1); i < QDP_Nc; i++) {
      QLA_Complex t2;
      t2 = QLA_elem_M(tm,j,i);
      for(c = 0; c < j; c++)
        QLA_c_meq_c_times_c(t2, QLA_elem_M(tm,c,i), QLA_elem_M(tm,j,c));

      QLA_c_eq_c_div_c(QLA_elem_M(tm,j,i), t2, QLA_elem_M(tm,j,j));
    }
  }

  /* The determinant */
  z1 = QLA_elem_M(tm,0,0);
  for(c = 1; c < QDP_Nc; c++) {
    QLA_Complex z;
    QLA_c_eq_c_times_c(z, z1, QLA_elem_M(tm,c,c));
    z1 = z;
  }

  return z1;
}


static void
normalize(QLA_ColorMatrix *m, int r)
{
  QLA_Real n, t;
  int c;

  n = 0;
  for(c=0; c<QDP_Nc; c++) {
    QLA_R_eq_norm2_C(&t, &QLA_elem_M(*m, r, c));
    n += t;
  }
  n = 1/sqrt(n);
  for(c=0; c<QDP_Nc; c++) {
    QLA_C_eq_r_times_C(&QLA_elem_M(*m,r,c), &n, &QLA_elem_M(*m,r,c));
  }
}

static void
orthogonalize(QLA_ColorMatrix *m, int r1, int r2)
{
  QLA_Complex z, t;
  int c;

  QLA_C_eq_zero(&z);
  for(c=0; c<QDP_Nc; c++) {
    QLA_C_eq_C_dot_C(&t, &QLA_elem_M(*m,r1,c), &QLA_elem_M(*m,r2,c));
    QLA_C_peq_C(&z, &t);
  }
  for(c=0; c<QDP_Nc; c++) {
    QLA_C_meq_C_times_C(&QLA_elem_M(*m,r2,c), &z, &QLA_elem_M(*m,r1,c));
  }
}


static void
make_unitary_func(QLA_ColorMatrix *m, int coords[])
{
  QLA_Complex z1, z2;
  QLA_Real r;
  int i, j, c;

  for(i=0; i<QDP_Nc; i++) {
    for(j=0; j<i; j++) {
      orthogonalize(m, j, i);
    }
    normalize(m, i);
  }

  z1 = det(m);

  r = QLA_norm_c(z1);
  QLA_c_eq_ca(z2, z1);
  QLA_c_eq_c_div_r(z1, z2, r);
  for(c = 0; c < QDP_Nc; ++c) {
    QLA_Complex z;
    QLA_c_eq_c_times_c(z, QLA_elem_M(*m,c,QDP_Nc-1), z1);
    QLA_elem_M(*m,c,QDP_Nc-1) = z;
  }
}

void 
random_link(QDP_ColorMatrix *link, QDP_RandomState *rs, QDP_Subset subset)
{
  QDP_ColorMatrix *cm = QDP_create_M();
  QLA_Real scale = SCALE;
  QLA_Complex z;

  QLA_c_eq_r(z, 1.0);
  QDP_M_eq_c(link, &z, subset);
  QDP_M_eq_gaussian_S(cm, rs, subset);
  QDP_M_peq_r_times_M(link, &scale, cm, subset);
  QDP_destroy_M(cm);
  QDP_M_eq_func(link, make_unitary_func, subset);
}
