#include <qdp-config.h>

int lattice_size[NDIM] = { 8, 8, 8, 16 };

int
main(int argc, char *argv[])
{
  int status = 1;
  int mu;
  QDP_ColorMatrix *U[NDIM];
  QLA_Real plaq;

  /* start QDP */
  QDP_initialize(&argc, &argv);

  /* set lattice size and create layout */
  QDP_set_latsize(NDIM, lattice_size);
  QDP_create_layout();

  /* one argument is the gauge file name */
  if (argc != 2) {
    printf0("ERROR: usage: plaquette gauge-file\n");
    goto end;
  }

  /* allocate the gauge field */
  for (mu = 0; mu < NDIM; mu++) {
    U[mu] = QDP_create_M();
  }

  /* read gauge field */
  if (read_gauge(U, argv[1]) != 0) {
    printf0("ERROR: read_gauge(%s)\n", argv[1]);
    goto end;
  }

  /* Compute plaquette */
  plaq = plaquette(U);

  /* delete the gauge field */
  for (mu = 0; mu < NDIM; mu++) QDP_destroy_M(U[mu]);  

  /* Display the value */
  printf0("plaquette{%s} = %g\n", argv[1],
           plaq / (QDP_volume() * QDP_Nc * NDIM * (NDIM - 1) / 2 ));

  status = 0;
end:
  /* shutdown QDP */
  QDP_finalize();

  return status;
}
