#include <qdp-config.h>

int lattice_size[NDIM] = { 8, 8, 8, 16 };

int
main(int argc, char *argv[])
{
  int status = 1;
  int mu;
  QDP_ColorMatrix *force;
  QDP_ColorMatrix *link;
  QDP_ColorMatrix *U[NDIM];
  QDP_RandomState *rs;
  QLA_Real plaq;

  /* start QDP */
  QDP_initialize(&argc, &argv);

  /* set lattice size and create layout */
  QDP_set_latsize(NDIM, lattice_size);
  QDP_create_layout();

  /* one argument is the gauge file name */
  if (argc != 3) {
    printf0("ERROR: usage: plaquette input output\n");
    goto end0;
  }
  /* allocate the gauge field and extra SU(3) variables */
  for (mu = 0; mu < NDIM; mu++) {
    U[mu] = QDP_create_M();
  }
  force = QDP_create_M();
  link = QDP_create_M();

  /* allocate and initialize random state */
  rs = QDP_create_S();
  init_random(rs, 32582657);

  /* read gauge field */
  if (read_gauge(U, argv[1]) != 0) {
    printf0("ERROR: read_gauge(%s)\n", argv[1]);
    goto end;
  }

  /* Compute plaquette */
  plaq = plaquette(U);

  /* Display the value */
  printf0("initial plaquette = %g\n",
           plaq / (QDP_volume() * QDP_Nc * NDIM * (NDIM - 1) / 2 ));

  /* random link at even sites */
  random_link(force, rs, QDP_even);

  /* update U[0] at even sites */
  QDP_M_eq_M_times_M(link, force, U[0], QDP_even);
  QDP_M_eq_M(U[0], link, QDP_even);

  /* compute and print the new plaquette */
  plaq = plaquette(U);
  printf0("new plaquette     = %g\n",
          plaq / (QDP_volume() * QDP_Nc * NDIM * (NDIM - 1) / 2));

  /* write U to a new file */
  if (write_gauge(argv[2], U) != 0) {
    printf("ERROR: write_gauge(%s)\n", argv[2]);
    goto end;
  }

  status = 0;
end:
  /* remove all allocated QDP objects */
  QDP_destroy_S(rs);
  QDP_destroy_M(link);
  QDP_destroy_M(force);
  for (mu = 0; mu < NDIM; mu++) QDP_destroy_M(U[mu]);
end0:
  /* shutdown QDP */
  QDP_finalize();

  return status;
}
