#include <qdp-config.h>

/* initialize the QOP library */
void
wilson_invert_init(void)
{
  QOP_layout_t qoplayout;
  qoplayout.latdim = QDP_ndim();
  qoplayout.latsize = (int *) malloc(QDP_ndim()*sizeof(int));
  qoplayout.machdim = -1;
  QDP_latsize(qoplayout.latsize);
  printf0("begin init\n");
  QOP_init(&qoplayout);
}

/* helper function to negate link matrix on last time slice */
static void
change_bc(QLA_ColorMatrix *cm, int coords[])
{
  if(coords[3]==QDP_coord_size(3)-1) QLA_M_eqm_M(cm, cm);
}

QOP_FermionLinksWilson *
wilson_load_links(QDP_ColorMatrix *U[])
{
  QOP_FermionLinksWilson *flw;
  QOP_opt_t opt[4];

  /* set some optional optimization parameters */
  opt[0].tag = "st";
  opt[0].value = 3;
  opt[1].tag = "ns";
  opt[1].value = 8;
  opt[2].tag = "nm";
  opt[2].value = 8;
  opt[3].tag = "cg";
  opt[3].value = 0;
  QOP_wilson_invert_set_opts(opt, 4);
  QDP_set_block_size(32);

  /* set antiperiodic boundary conditions in time direction */
  QDP_M_eq_func(U[3], change_bc, QDP_all);

  /* create wilson fermions links */
  flw = QOP_wilson_create_L_from_qdp(U, NULL);

  /* remove antiperiodic boundary conditions in time direction */
  QDP_M_eq_func(U[3], change_bc, QDP_all);

  return flw;
}

void
wilson_invert(QDP_DiracFermion *prop, QOP_FermionLinksWilson *flw,
              QDP_DiracFermion *source, QLA_Real kappa, double res)
{
  QOP_invert_arg_t inv_arg;
  QOP_resid_arg_t res_arg;
  QOP_DiracFermion *qopout, *qopin;
  QOP_info_t info;

  inv_arg.max_iter = 1000;
  inv_arg.restart = 500;
  inv_arg.max_restarts = 5;
  inv_arg.evenodd = QOP_EVENODD;
  res_arg.rsqmin = res*res;

  qopout = QOP_create_D_from_qdp(prop);
  qopin = QOP_create_D_from_qdp(source);

  QOP_wilson_invert(&info, flw, &inv_arg, &res_arg, kappa, qopout, qopin);

  printf0("iters = %4i  secs = %8f  mflops = %.0f\n", res_arg.final_iter,
          info.final_sec, 1e-6*info.final_flop/info.final_sec);

  QOP_extract_D_to_qdp(prop, qopout);

  QOP_destroy_D(qopout);
  QOP_destroy_D(qopin);
}
