#include <qdp-config.h>

int lattice_size[NDIM] = { 8, 8, 8, 16 };

int
main(int argc, char *argv[])
{
  int status = 1;
  int mu, color, spin;
  int coord[NDIM];
  QDP_ColorMatrix *U[NDIM];
  QLA_Real plaq, kappa;
  double resid;
  QOP_FermionLinksWilson *wilson_links;
  QDP_DiracFermion *source, *soln;
  QDP_DiracPropagator *prop;

  /* start QDP */
  QDP_initialize(&argc, &argv);

  /* set lattice size and create layout */
  QDP_set_latsize(NDIM, lattice_size);
  QDP_create_layout();

  /* one argument is the gauge file name */
  if (argc != 3) {
    printf0("ERROR: usage: rho gauge-file kappa\n");
    goto end;
  }

  /* allocate the gauge field */
  for (mu = 0; mu < NDIM; mu++) {
    U[mu] = QDP_create_M();
  }

  /* read gauge field */
  if (read_gauge(U, argv[1]) != 0) {
    printf0("ERROR: read_gauge(%s)\n", argv[1]);
    goto end;
  }

  /* Compute and print plaquette as a check */
  plaq = plaquette(U);
  printf0("plaquette{%s} = %g\n", argv[1],
           plaq / (QDP_volume() * QDP_Nc * NDIM * (NDIM - 1) / 2 ));

  /* initialize Wilson Dirac matrix solver */
  wilson_invert_init();

  /* load gauge field */
  wilson_links = wilson_load_links(U);

  /* set source point at 0,0,0,0 */
  for(mu=0; mu<NDIM; mu++) coord[mu] = 0;

  /* create some fields */
  source = QDP_create_D();
  soln = QDP_create_D();
  prop = QDP_create_P();

  kappa = atof(argv[2]);
  resid = 1e-7;
  printf0("Kappa = %g\n", kappa);

  /* do inversions on source point for each color and spin */
  for(color = 0; color < QLA_Nc; color++) {
    for(spin = 0; spin < QLA_Ns; spin++) {
      point_source(source, coord, color, spin);
      QDP_D_eq_zero(soln, QDP_all);
      wilson_invert(soln, wilson_links, source, kappa, resid);
      QDP_P_eq_diracvec_D(prop, soln, color, spin, QDP_all);
    }
  }

  /* print out rho correlator */
  print_rho(prop, coord[3]);

  /* free some fields */
  QDP_destroy_D(source);
  QDP_destroy_D(soln);
  QDP_destroy_P(prop);

  /* delete the gauge field */
  for (mu = 0; mu < NDIM; mu++) QDP_destroy_M(U[mu]);  

  status = 0;
end:
  /* shutdown QDP */
  QDP_finalize();

  return status;
}
