#include <qdp-config.h>

static QDP_Subset *timeslices = NULL;

static int
timeslice_func(int *coord, void *dum)
{
  return coord[NDIM-1];
}

void
print_rho(QDP_DiracPropagator *prop, int t0)
{
  int i, nt, gamma;
  QLA_Complex *corr;
  QDP_DiracPropagator *prop_gamma, *gamma_prop_gamma;

  /* get number of timeslices */
  nt = QDP_coord_size(NDIM-1);

  /* create subset for each timeslice */
  if(!timeslices) {
    timeslices = QDP_create_subset(timeslice_func, NULL, 0, nt);
  }

  /* allocate space for correlator on each timeslice */
  corr = (QLA_Complex *) malloc(nt*sizeof(QLA_Complex));

  /* allocate temporary propagators */
  prop_gamma = QDP_create_P();
  gamma_prop_gamma = QDP_create_P();

  /* gamma_1 gamma_5 = g_2 g_3 g_4  ->  1110_2 = 15 - 1 */
  gamma = 15 - 1;

  /* multiply prop by GAMMA on both sides */
  QDP_P_eq_P_times_gamma(prop_gamma, prop, gamma, QDP_all);
  QDP_P_eq_gamma_times_P(gamma_prop_gamma, prop_gamma, gamma, QDP_all);

  /* do sum[trace(prop^+ GAMMA prop GAMMA)] over each timeslice */
  QDP_c_eq_P_dot_P_multi(corr, prop, gamma_prop_gamma, timeslices, nt);

  /* print result */
  printf0("BEGIN RHO\n");
  for(i=0; i<nt; i++) {
    int t = (t0+i)%nt;
    printf0("%i\t%g\n", i, -QLA_real(corr[t]) );
  }
  printf0("END RHO\n");

  free(corr);
  QDP_destroy_P(prop_gamma);
  QDP_destroy_P(gamma_prop_gamma);
}
